/*
** Vinculo_GLCD.c
**
** Copyright  2010 Future Devices International Limited
**
** C Source file for Vinculum II test applications
** Main module
**
** Author: FTDI - P. Rybak
** Project: Vinculum II / Vinculo
** Module: Vinculum II Applications
** Requires: VOS, DMA, IOMux, SPIMaster, GPIO
** Comments:
**
** History:
**  1.0  Initial version
**
*/

#include "vos.h"
#include "DMA.h"
#include "IOMux.h"
#include "GPIO.h"
#include "SPIMaster.h"

#include "Vinculo_GLCD.h"
#include "LM6059BCW.h"
#include "Font_5x7.h"
#include "Font_8x8.h"
#include "V2Logo.h"


// VOS TCB pointers for threads
vos_tcb_t        *tcbFirmware;

// VOS handles for opened devices
VOS_HANDLE hSPIm, hGpioB;

// SPI Master IOCTL request block
common_ioctl_cb_t spim_iocb;


// forward declarations
void firmware(void);

unsigned char lcd_buffer[128];
unsigned char cmd_buffer[8];

/* main
*
* This is the main application called by VOS at startup. It is responsible
* for device and thread initialisation.
*
*/
void main(void)
{
    // GPIO context structure
    gpio_context_t gpioCtx;
    // SPI Master context structure
    spimaster_context_t spimCtx;

    // call VOS initialisation routines
    vos_init(10, VOS_TICK_INTERVAL, NUMBER_OF_DEVICES);
    vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);

    // Setup IOMUX
    // Control port = GPIO PORT_B
    vos_iomux_define_output(63,IOMUX_OUT_GPIO_PORT_B_2); // PortB_2->LCD_RST#
    vos_iomux_define_output(13,IOMUX_OUT_GPIO_PORT_B_6); // PortB_6->LCD_A0
    // SPI pins
    vos_iomux_define_output(14,IOMUX_OUT_SPI_MASTER_CS_0); // LCD_CS#
    vos_iomux_define_output(15,IOMUX_OUT_SPI_MASTER_CLK);  // LCD_CLK
    vos_iomux_define_output(16,IOMUX_OUT_SPI_MASTER_MOSI); // LCD_SDI

    // initialise device drivers
    spimCtx.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
    spimaster_init(VOS_DEV_SPIM,&spimCtx);

    gpioCtx.port_identifier = GPIO_PORT_B;
    gpio_init(VOS_DEV_GPIOB,&gpioCtx);

    // create threads for firmware application (no parameters)
    tcbFirmware = vos_create_thread(29, SIZEOF_THREAD_MEMORY, firmware, 0);
    // start VOS scheduler
    vos_start_scheduler();

main_loop:
    goto main_loop;
}
//---------------------------------------------------------
// LCD functions
//---------------------------------------------------------
void write_lcd_data(unsigned char *data, unsigned char len)
{
    unsigned char ctrl_byte;

    ctrl_byte = LCD_A0|LCD_RST;
    vos_dev_write(hGpioB,&ctrl_byte,1,NULL); // Set A0 and RESET line high
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_ENABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    vos_dev_write(hSPIm, data, (len&0x00FF), NULL);
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);
}
//---------------------------------------------------------
void write_lcd_cmd(unsigned char *cmd, unsigned char len)
{
    unsigned char ctrl_byte;

    ctrl_byte = (~LCD_A0)|LCD_RST;
    vos_dev_write(hGpioB,&ctrl_byte,1,NULL); // Set A0 and RESET line high
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_ENABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    vos_dev_write(hSPIm, cmd, (len&0x00FF), NULL);
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);
}
//---------------------------------------------------------
void lcd_on(void)
{
    cmd_buffer[0] = CMD_DISP_ON; // Send 'Display On' command
    write_lcd_cmd(cmd_buffer, 1);
}
//---------------------------------------------------------
void lcd_off(void)
{
    cmd_buffer[0] = CMD_DISP_OFF; // Send 'Display Off' command
    write_lcd_cmd(cmd_buffer, 1);
}
//---------------------------------------------------------
void lcd_set_contrast(unsigned char contrast_value)
{
    cmd_buffer[0] = CMD_DISP_EVOLUME_MODE; // Send 'Set Reference Voltage' command
    cmd_buffer[1] = contrast_value;
    write_lcd_cmd(cmd_buffer, 2);
}
//---------------------------------------------------------
void SetPageAddress(unsigned char page)
{
    cmd_buffer[0] = CMD_DISP_PAGE_ADDR | (page &0x0F);
    write_lcd_cmd(cmd_buffer, 1);
}
//---------------------------------------------------------
void SetColumnAddress(unsigned char column)
{
    cmd_buffer[0] = CMD_DISP_COLADDR_H | (column>>4);
    cmd_buffer[1] = CMD_DISP_COLADDR_L | (column&0x0F);
    write_lcd_cmd(cmd_buffer, 2);
}
//---------------------------------------------------------
void SetLineAddress(unsigned char line)
{
    cmd_buffer[0] = CMD_DISP_LINE_ADDR | line;
    write_lcd_cmd(cmd_buffer, 1);
}
//---------------------------------------------------------
void lcd_clear(void)
{
    unsigned char i;

    for(i=0;i<128;i++)
    {
        lcd_buffer[i]=0;
    }
    for(i=0;i<8;i++)
    {
        SetPageAddress(i);
        SetColumnAddress(0);
        SetLineAddress(0);
        write_lcd_data(lcd_buffer, 128);
    }
}
//---------------------------------------------------------
void lcd_clear_line(unsigned char y)
{
    unsigned char i;

    for(i=0;i<128;i++)
    {
        lcd_buffer[i]=0;
    }
    SetPageAddress(y);
    SetColumnAddress(0);
    write_lcd_data(lcd_buffer, 128);
}
//---------------------------------------------------------
void lcd_ini(void)
{
    unsigned char data_byte;

    // Hardware reset
    data_byte = (0xFF&(~LCD_RST));
    vos_dev_write(hGpioB,&data_byte,1,NULL); // LCD Reset line - low
    vos_delay_msecs(5);
    data_byte = (data_byte|LCD_RST);
    vos_dev_write(hGpioB,&data_byte,1,NULL); // LCD Reset line - high
    vos_delay_msecs(5);

    cmd_buffer[0] = CMD_DISP_SET_BIAS; // Send 'Set Bias' command
    cmd_buffer[1] = CMD_DISP_SCANDIR; // Send 'ADC Normal' command
    cmd_buffer[2] = CMD_DISP_REV|0x08; // Send 'Flip on Y' command
    cmd_buffer[3] = CMD_DISP_LINE_ADDR; // Send 'Line addr = 0' command
    write_lcd_cmd(cmd_buffer, 4);

    cmd_buffer[0] = CMD_DISP_PWRCTRL|LCD_VCNV; // Send 'Power Ctrl' command
    write_lcd_cmd(cmd_buffer, 1);
    vos_delay_msecs(50);
    cmd_buffer[0] = cmd_buffer[0]|LCD_VREG;
    write_lcd_cmd(cmd_buffer, 1);
    vos_delay_msecs(50);
    cmd_buffer[0] = cmd_buffer[0]|LCD_VFOL;
    write_lcd_cmd(cmd_buffer, 1);
    vos_delay_msecs(50);

    cmd_buffer[0] = CMD_DISP_VREG_RES_RATIO|0x06; // Send 'Regulator Resistor Select' command
    cmd_buffer[1] = CMD_DISP_EVOLUME_MODE; // Send 'Set Reference Voltage' command
    cmd_buffer[2] = LCD_EVOLUME_VALUE;
    cmd_buffer[3] = CMD_DISP_ON; // Send 'Display On' command
    write_lcd_cmd(cmd_buffer, 4);
    vos_delay_msecs(50);

    lcd_clear();

    SetPageAddress(0);
    SetColumnAddress(0);
    SetLineAddress(0);
}
//---------------------------------------------------------
void show_logo(unsigned short delay1)
{
    unsigned char i,j;
    unsigned short temp;

    SetPageAddress(0);
    SetColumnAddress(0);
    for (j=0;j<8;j++)
    {
        SetPageAddress(j);
        SetColumnAddress(0);
        for (i=0;i<128;i++)
        {
            temp=128*j;
            temp+=i;
            lcd_buffer[i]= img[temp];
        }
        write_lcd_data(lcd_buffer,128);
        vos_delay_msecs(delay1);
    }
}
//---------------------------------------------------------
// This function will display string on one line of the display.
// If string bitmap is longer than 128 bytes it will be truncated.
// Character bitmap size is 8x8 pixels.
void LCDprintf8x8(unsigned char x,unsigned char y,unsigned char inv,unsigned char *Str)
{
    unsigned char m,n;
    unsigned short yy;

    SetPageAddress(y); // Line where to display the string (0..7)
    SetColumnAddress(x);  // Horizontal offset (0..127)
    n=x;
    while(*Str != '\0') // While not the end of the string
    {
        if((n+8)>127) // Check if one more character can be displayed
        {
            break;
        }
        else
        {
            yy = *Str; // Get ASCII value
            yy = (yy*8);  // Calculate the position of charecter bitmap
            for(m=0; m<8; m++)
            {
                lcd_buffer[n]= font8x8[yy+m]; // Copy character bitmap to display buffer
                if(inv)
                {
                    lcd_buffer[n]= (~lcd_buffer[n]); // Copy character bitmap to display buffer
                }
                n++;
            }
            ++Str; // Increment the pointer
        }
    }
    write_lcd_data(lcd_buffer, (n+1)); // Send buffer to the display.
}

//---------------------------------------------------------
// This function will display string on one line of the display.
// If string bitmap is longer than 128 bytes it will be truncated.
// Character bitmap size is 5x7 pixels.
void LCDprintf5x7(unsigned char x,unsigned char y,unsigned char inv,unsigned char *Str)
{
    unsigned char m,n;
    unsigned short yy;

    SetPageAddress(y); // Line where to display the string (0..7)
    SetColumnAddress(x);  // Horizontal offset (0..127)
    n=x;
    while(*Str != '\0') // While not the end of the string
    {
        if((n+5)>127) // Check if one more character can be displayed
        {
            break;
        }
        else
        {
            yy = *Str; // Get ASCII value
            yy = (yy-32)*5;  // Calculate the position of charecter bitmap
            for(m=0; m<5; m++)
            {
                lcd_buffer[n]= font5x7[yy+m]; // Copy character bitmap to display buffer
                if(inv)
                {
                    lcd_buffer[n]= (~lcd_buffer[n]); // Copy character bitmap to display buffer
                }
                n++;
            }
            ++Str; // Increment the pointer
        }
    }
    write_lcd_data(lcd_buffer, (n+1)); // Send buffer to the display.
}
//---------------------------------------------------------
/* firmware
*
* This is the thread created by the main application.
*/
void firmware(void)
{

    // GPIO IOCTL request block
    gpio_ioctl_cb_t gpio_iocb;

    // general purpose variables
    unsigned char i;

    // find and open GPIO device port B
    hGpioB = vos_dev_open(VOS_DEV_GPIOB);

    gpio_iocb.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
    gpio_iocb.value = 0x44;    // set bit 2 and 6 as outputs, all other as inputs
    vos_dev_ioctl(hGpioB, &gpio_iocb);

    //*********************************************************
    // INITIALISE SPI MASTER PARAMETERS
    //*********************************************************

    // open SPI Master and get a handle
    hSPIm = vos_dev_open(VOS_DEV_SPIM);

    // enable DMA
    spim_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // set clock phase
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SCK_CPHA;
    spim_iocb.set.param = SPI_MASTER_SCK_CPHA_1;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // set clock polarity
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SCK_CPOL;
    spim_iocb.set.param = SPI_MASTER_SCK_CPOL_1;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // set data order
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_DATA_ORDER;
    spim_iocb.set.param = SPI_MASTER_DATA_ORDER_MSB;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // set clock rate
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SET_SCK_FREQUENCY;
    spim_iocb.set.spi_master_sck_freq = 1000000;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SET_DATA_DELAY;
    spim_iocb.set.param = 0;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // set chip select
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_1;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    lcd_ini();

    while(1)
    {
        lcd_clear();
        show_logo(200);
        vos_delay_msecs(2000);
        i = LCD_EVOLUME_VALUE;
        while(i>0)
        {
            lcd_set_contrast(i);
            i--;
            vos_delay_msecs(100);

        }
        lcd_off();
        lcd_clear();
        LCDprintf8x8(0,1,0,LINE1);
        LCDprintf8x8(0,2,0,LINE2);
        LCDprintf5x7(0,3,0,LINE3);
        LCDprintf5x7(0,4,0,LINE4);
        lcd_on();
        i=0;
        while(i<(LCD_EVOLUME_VALUE+1))
        {
            lcd_set_contrast(i);
            i++;
            vos_delay_msecs(100);

        }
        vos_delay_msecs(1000);
        for(i=0;i<3;i++)
        {
            lcd_off();
            vos_delay_msecs(330);
            lcd_on();
            vos_delay_msecs(660);
        }
        // Shift display 8 lines up
        for(i=0;i<8;i++)
        {
            SetLineAddress(i);
            vos_delay_msecs(200);
        }
        lcd_clear_line(1);
        LCDprintf5x7(0,0,0,LINE5);
        // Shift display 8 lines up
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+8);
            vos_delay_msecs(200);
        }
        lcd_clear_line(2);
        LCDprintf5x7(0,1,0,LINE6);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+16);
            vos_delay_msecs(200);
        }
        lcd_clear_line(3);
        LCDprintf5x7(0,2,0,LINE7);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+24);
            vos_delay_msecs(200);
        }
        lcd_clear_line(4);
        LCDprintf5x7(0,3,0,LINE8);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+32);
            vos_delay_msecs(200);
        }
        lcd_clear_line(5);
        LCDprintf5x7(0,4,0,LINE9);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+40);
            vos_delay_msecs(200);
        }
        lcd_clear_line(6);
        LCDprintf5x7(0,5,0,LINE10);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+48);
            vos_delay_msecs(200);
        }
        lcd_clear_line(7);
        LCDprintf5x7(0,6,0,LINE11);
        for(i=0;i<8;i++)
        {
            SetLineAddress(i+56);
            vos_delay_msecs(200);
        }
        lcd_clear_line(0);
        LCDprintf5x7(0,7,0,LINE12);

        vos_delay_msecs(2000);

    }
}

